#include "ecpri_timer.h"

#include "ucp_ecpri.h"
#include "phy_para.h"
#include "mtimer_com.h"
#include "ucp_utility.h"
#include "ucp_printf.h"
#include "ucp_mtimer.h"
#include "inter_vector.h"
#include "ecs_rfm_dm_mgt.h"
#include "mtimer_drv.h"

extern stPhyScsPara* phyPara;
extern stMtimerIntStat gMtimerIntCnt[SCS_MAX_NUM];
extern stMtimerPhyPara gMtimerSfnNum[SCS_MAX_NUM];
extern stMtimerSfnCal gMtimerSfnCalPara[SCS_MAX_NUM];

void ecpri_init()
{
#ifdef PALLADIUM_TEST
	int32_t flag = 0;
	debug_write((DBG_DDR_IDX_DRV_BASE+68), flag);	// 0x110
#endif

	init_ecpri_clk();
#ifdef PALLADIUM_TEST
	flag++;
	debug_write((DBG_DDR_IDX_DRV_BASE+68), flag);	// 0x110
#endif

	init_ecpri_pma_rst();
#ifdef PALLADIUM_TEST
	flag++;
	debug_write((DBG_DDR_IDX_DRV_BASE+68), flag);	// 0x110
#endif

	ecpri_pcs_rst_ctrl();
#ifdef PALLADIUM_TEST
	flag++;
	debug_write((DBG_DDR_IDX_DRV_BASE+68), flag);	// 0x110
#endif

	ecpri_timer_init(NR_SCS_60K, 20);
#ifdef PALLADIUM_TEST
	flag++;
	debug_write((DBG_DDR_IDX_DRV_BASE+68), flag);	// 0x110
#endif
}

void init_ecpri_clk(void)
{
	JECS_CRG_PLLSEL |=BIT11+BIT4+BIT5+BIT31;
	JECS_CRG_CLK_CTRL6		 =	 0x521000; //SYSAPB /2	apb_aclk_i
	//switch_core_clk_i,
	//switch_sch_clk_i,
	JECS_CRG_CLK_CTRL7		 =	 0x520000;//jecs_pma_tx0_ropll_div33_clk
	//ecpri_switch_mae_core_clk
	JECS_CRG_CLK_CTRL8		=	 0x520000;//jecs_pma_tx0_ropll_div33_clk
	//ecpri_pcs_xgmii_tx_clk
	JECS_CRG_CLK_CTRL9		=	 0x521000;//jecs_pma_tx0_ropll_div33_clk
	//ecpri_pcs_xgmii_rx_clk
	JECS_CRG_CLK_CTRL0		=	 0x521000;//jecs_pma_rx0_div33_clk
}

void init_ecpri_pma_rst(void)
{
  //cancell jecs pma rst
  JECS_CRG_PMA16_RST_CTRL |= BIT24;
  JECS_CRG_PMA8_RST_CTRL  |= BIT24;
  JECS_CRG_PMA9_RST_CTRL  |= BIT24;
  JECS_CRG_PMA10_RST_CTRL |= BIT24;
  JECS_CRG_PMA11_RST_CTRL |= BIT24;
  JECS_CRG_PMA12_RST_CTRL |= BIT24;
  JECS_CRG_PMA13_RST_CTRL |= BIT24;
  JECS_CRG_PMA14_RST_CTRL |= BIT24;
  JECS_CRG_PMA15_RST_CTRL |= BIT24;
}

void ecpri_pcs_rst_ctrl()
{
  //cancell ecpri's rst
  JECS_CRG_MANTICORE0_RST_CTRL      |= BIT24;
  JECS_CRG_MANTICORE1_RST_CTRL      |= BIT24;
  JECS_CRG_MANTICORE2_RST_CTRL      |= BIT24;
  JECS_CRG_MANTICORE3_RST_CTRL      |= BIT24;
  JECS_CRG_MANTICORE4_RST_CTRL      |= BIT24;
  JECS_CRG_MANTICORE5_RST_CTRL      |= BIT24;
  JECS_CRG_MANTICORE6_RST_CTRL      |= BIT24;
  JECS_CRG_MANTICORE7_RST_CTRL      |= BIT24;
  JECS_CRG_MANTICORE8_RST_CTRL      |= BIT24;
  ECPRI_RST_CFG_REG                 |= BIT24;
}

void ecpri_mtimer_init(int32_t nScsId, int32_t nTddSlotNum)
{
	stMtimerIntStat* pMtimerInt = &gMtimerIntCnt[MTIMER_ECPRI_ID];
	stMtimerPhyPara* pMtimerSfn = &gMtimerSfnNum[MTIMER_ECPRI_ID];
	stMtimerSfnCal* pMtimerCalPara = &gMtimerSfnCalPara[MTIMER_ECPRI_ID];
	
	memset(pMtimerInt, 0, sizeof(stMtimerIntStat));
	memset(pMtimerSfn, 0, sizeof(stMtimerPhyPara));
	memset(pMtimerCalPara, 0, sizeof(stMtimerSfnCal));
	
	mtimer_para_init(MTIMER_ECPRI_ID, nScsId, nTddSlotNum);
	
    ecpri_timer_init(nScsId, nTddSlotNum);
}

void ecpri_timer_init(int32_t nScsId, int32_t nTddSlotNum)
{
    //EcsRfmDmLocalMgt_t* pEcsDmLocalMgt = get_ecs_rfm_dm_local_mgt();
    //uint32_t coreClk = pEcsDmLocalMgt->pCpriPara->core_clk;

	mtimer_clear_all_event(MTIMER_ECPRI_ID);
	set_ecpri_tmr_period();
	set_ecpri_timer_int();
}

void ecpri_timer_reconfig(phy_timer_config_ind_t *my_ecpritmr)
{
    EcsRfmDmLocalMgt_t* pEcsDmLocalMgt = get_ecs_rfm_dm_local_mgt();
	stMtimerPara* pMtimerPara = pEcsDmLocalMgt->pMtimerPara[MTIMER_ECPRI_ID];
	stMtimerPhyPara* pMtimerSfn = &gMtimerSfnNum[MTIMER_ECPRI_ID];
	
	pMtimerPara->scsId = my_ecpritmr->scsId;
	pMtimerPara->tddPeriod = my_ecpritmr->t_period; // us
	pMtimerPara->tddSlotNum = my_ecpritmr->num_t;
	pMtimerPara->slotPeriod = my_ecpritmr->t_us;
	pMtimerPara->slotMaxNum = my_ecpritmr->num_t_per_sfn;
	pMtimerSfn->slotMaxNum = my_ecpritmr->num_t_per_sfn;

	//gScsId = my_ecpritmr->scsId;
	//gMtimerId = MTIMER_ECPRI_ID;

	//gCellSfnPara[MTIMER_ECPRI_ID].scsId = my_ecpritmr->scsId;

	do_write_short((&(phyPara[my_ecpritmr->scsId].slotNumOfTdd)), my_ecpritmr->num_t);
	do_write_short((&(phyPara[my_ecpritmr->scsId].mtimerId)), MTIMER_ECPRI_ID);
	__ucps2_synch(0);

	set_ecpri_tmr_period();					// set OVF value, every slot int and 10ms int, cevent0/2 for ape0, report link status

	set_ecpri_tdd_offset();					// 5ms int, tevent2 for all ape cores, dma

	set_ecpri_tx_slot_offset();
	set_ecpri_rx_slot_offset();
}

void set_ecpri_tmr_period(void)
{
    EcsRfmDmLocalMgt_t* pEcsDmLocalMgt = get_ecs_rfm_dm_local_mgt();
	stMtimerPara* pMtimerPara = pEcsDmLocalMgt->pMtimerPara[MTIMER_ECPRI_ID];
	
	// 设置计数器溢出值
	uint32_t tempL = pMtimerPara->tmrMsPeriod * pMtimerPara->slotPeriod / 1000;
	uint32_t tempM = 2; //gCpriTimerPara.tddSlotNum;
	uint32_t tempH = SFN_PERIOD / pMtimerPara->slotPeriod / 2; //gCpriTimerPara.tddSlotNum;

	set_mtimer_period(MTIMER_ECPRI_ID, tempL, tempM, tempH);
}

void set_ecpri_tdd_offset(void)
{
    EcsRfmDmLocalMgt_t* pEcsDmLocalMgt = get_ecs_rfm_dm_local_mgt();
	stMtimerPara* pMtimerPara = pEcsDmLocalMgt->pMtimerPara[MTIMER_ECPRI_ID];
	uint32_t offset = 0;
#ifdef DISTRIBUTED_BS
	offset = pEcsDmLocalMgt->pCpriDelay->cpri10msOffset;
#endif	
	uint32_t tmr2Point = pMtimerPara->tddPeriod - offset;  // us // 5ms
	set_mtimer_tmrpoint(MTIMER_ECPRI_ID, MTMR_TDD_OFFSET, tmr2Point, MTIMER_MASK_48BIT);
	enable_mtimer_tmrpoint_int(MTIMER_ECPRI_ID, MTMR_TDD_OFFSET, MTMR_INT_TDD_OFFSET);
}

void set_ecpri_tx_slot_offset()
{
    //EcsRfmDmLocalMgt_t* pEcsDmLocalMgt = get_ecs_rfm_dm_local_mgt();
	
	uint32_t tmr3Point = SFN_PERIOD - INT_DELAY; // us
	set_mtimer_tmrpoint(MTIMER_ECPRI_ID, MTMR_TXSLOT_OFFSET, tmr3Point, MTIMER_MASK_32BIT);
	enable_mtimer_tmrpoint_int(MTIMER_ECPRI_ID, MTMR_TXSLOT_OFFSET, MTMR_INT_SLOT_OFFSET);
	
	//uint32_t tempL = gCpriTimerPara.tmrMsPeriod * (tmr3Point % gCpriTimerPara.slotPeriod) / 1000;
	//uint32_t addr = (uint32_t)&(phyPara[gCpriTimerPara.scsId].txSetVal);
	//do_write(addr, tempL);

	// ape tmrpoints
	for (int32_t i = 0; i < APE_NUM; i++)
	{
		int32_t tmrId = MTMR_APE0_TXSLOT + (i<<1);
		set_mtimer_tmrpoint(MTIMER_ECPRI_ID, tmrId, tmr3Point, MTIMER_MASK_32BIT);
		enable_mtimer_tmrpoint_int(MTIMER_ECPRI_ID, tmrId, (MTMR_INT_SLOT_OFFSET+i));
	}
}

void set_ecpri_rx_slot_offset()
{
    //EcsRfmDmLocalMgt_t* pEcsDmLocalMgt = get_ecs_rfm_dm_local_mgt();

	uint32_t tmr4Point = INT_DELAY; // us
	set_mtimer_tmrpoint(MTIMER_ECPRI_ID, MTMR_RXSLOT_OFFSET, tmr4Point, MTIMER_MASK_32BIT);
	enable_mtimer_tmrpoint_int(MTIMER_ECPRI_ID, MTMR_RXSLOT_OFFSET, MTMR_INT_SLOT_OFFSET);
	
	//uint32_t tempL = gCpriTimerPara.tmrMsPeriod * (tmr3Point % gCpriTimerPara.slotPeriod) / 1000;
	//uint32_t addr = (uint32_t)&(phyPara[gCpriTimerPara.scsId].txSetVal);
	//do_write(addr, tempL);

	// ape tmrpoints
	for (int32_t i = 0; i < APE_NUM; i++)
	{
		int32_t tmrId = MTMR_APE0_RXSLOT + (i<<1);
		set_mtimer_tmrpoint(MTIMER_ECPRI_ID, tmrId, tmr4Point, MTIMER_MASK_32BIT);
		enable_mtimer_tmrpoint_int(MTIMER_ECPRI_ID, tmrId, (MTMR_INT_SLOT_OFFSET+i));
	}
}

void set_ecpri_timer_int(void)
{
	int32_t apeId = get_core_id();

	uint32_t intNum = APC_ECPRI_SUB_INTR2;
	// attach interrupt func
	int32_t ret = smart_irq_request(intNum, isr_ecpri_timer);
	if (0 != ret)
	{
		UCP_PRINT_ERROR("attach int num: 0x%x error. errno = 0x%x. \r\n", intNum, ret);
		debug_write(DBG_DDR_ERR_IDX(apeId, 33), ret);
	}
}

void start_ecpri_timer(void)
{
	start_mtimer(MTIMER_ECPRI_ID);
}

void stop_ecpri_timer(void)
{
	stop_mtimer(MTIMER_ECPRI_ID);
}

void isr_ecpri_timer(void)
{
	uint32_t tmrIntcFlag = 0;
	uint32_t tEventFlag = 0;
	stMtimerIntStat* pMtimerInt = &gMtimerIntCnt[MTIMER_ECPRI_ID];
	stMtimerPhyPara* pMtimerSfn = &gMtimerSfnNum[MTIMER_ECPRI_ID];
	uint32_t tmrBaseAddr = mtimer_get_baseaddr(MTIMER_ECPRI_ID);

	pMtimerInt->tmrIntCnt++;
#ifdef PALLADIUM_TEST
	debug_write((DBG_DDR_IDX_DRV_BASE+123), pMtimerInt->tmrIntCnt);	// 0x1ec
#endif	
	tmrIntcFlag = do_read_volatile(tmrBaseAddr + MTMR_INTC_REG);

	uint32_t tFlagAddr = 0;
	if ((tmrIntcFlag & (1 << MTMR_INT_TDD_OFFSET)))  /* tmr int */
	{
		tFlagAddr = tmrBaseAddr + MTMR_TINTF00_REG + 6*(MTMR_INT_TDD_OFFSET<<2);
		tEventFlag = do_read_volatile(tFlagAddr);
		if (tEventFlag & (1<<MTMR_TDD_OFFSET))		// tdd offset int
		{
			do_write((tmrBaseAddr+MTMR_TEVENT0_REG), (1<<MTMR_TDD_OFFSET));
			do_write(tFlagAddr, (1<<MTMR_TDD_OFFSET));	// clear int flag
			pMtimerInt->tddOffsetIntCnt++;
			// start_csu_timing
#ifdef PALLADIUM_TEST
			debug_write((DBG_DDR_IDX_DRV_BASE+94), pMtimerInt->tddOffsetIntCnt); // 0x178
#endif
			tEventFlag = do_read_volatile(tFlagAddr);
		}
		do_write((tmrBaseAddr+MTMR_INTC_REG), (1 << MTMR_INT_TDD_OFFSET)); 		// clear int
	}

	if ((tmrIntcFlag & (1 << MTMR_INT_SLOT_OFFSET)))  /* tmr int */
	{
		tFlagAddr = tmrBaseAddr + MTMR_TINTF00_REG + 6*(MTMR_INT_SLOT_OFFSET<<2);
		tEventFlag = do_read_volatile(tFlagAddr);
		if (tEventFlag & ((1<<MTMR_TXSLOT_OFFSET)|(1<<MTMR_RXSLOT_OFFSET)))
		{
			if (tEventFlag & (1<<MTMR_TXSLOT_OFFSET)) // tx slot offset
			{
				do_write((tmrBaseAddr+MTMR_TEVENT0_REG), (1<<MTMR_TXSLOT_OFFSET));
				do_write(tFlagAddr, (1<<MTMR_TXSLOT_OFFSET));	// clear int flag
				pMtimerInt->txSlotIntCnt++;
				//uint32_t addr = 0;
#ifdef PALLADIUM_TEST
				debug_write((DBG_DDR_IDX_DRV_BASE+32), pMtimerInt->txSlotIntCnt);	// 0x80
#endif
				pMtimerSfn->txSlotNum++;
				__ucps2_synch(0);
				if (pMtimerSfn->txSlotNum == pMtimerSfn->slotMaxNum)
				{
					pMtimerSfn->txSfnNum++;
					pMtimerSfn->txSfnNum &= 0x3FF;
					pMtimerSfn->txSlotNum = 0;
				}
				__ucps2_synch(0);
#if 0 // def PALLADIUM_TEST
				debug_write(((DBG_DDR_IDX_DRV_BASE+448) + (gCpriTimerPara.txSlotIntCnt&0x3F)), gCpriTimerPara.txSlotIntCnt);	// 0x700
				debug_write(((DBG_DDR_IDX_DRV_BASE+512) + (gCpriTimerPara.txSlotIntCnt&0x3F)), get_tx_slot(1)); 				// 0x800
				nowTxStcCnt = GET_STC_CNT();
				debug_write(((DBG_DDR_IDX_DRV_BASE+576) + (gCpriTimerPara.txSlotIntCnt&0x3F)), nowTxStcCnt);					// 0x900
				debug_write(((DBG_DDR_IDX_DRV_BASE+640) + (gCpriTimerPara.txSlotIntCnt&0x3F)), (nowTxStcCnt-lastTxStcCnt)); 	// 0xa00
				lastTxStcCnt = nowTxStcCnt;
				
				debug_write(((DBG_DDR_IDX_DRV_BASE+8192) + (gCpriTxSlotIntCnt&0x1FF)), gCpriTxSlotIntCnt);		// 0x8000
				debug_write(((DBG_DDR_IDX_DRV_BASE+8704) + (gCpriTxSlotIntCnt&0x1FF)), get_tx_slot(1)); 		// 0x8800
				nowTxStcCnt = GET_STC_CNT();
				debug_write(((DBG_DDR_IDX_DRV_BASE+9216) + (gCpriTxSlotIntCnt&0x1FF)), nowTxStcCnt);			// 0x9000
				debug_write(((DBG_DDR_IDX_DRV_BASE+9728) + (gCpriTxSlotIntCnt&0x1FF)), (nowTxStcCnt-lastTxStcCnt)); 	// 0x9800
				lastTxStcCnt = nowTxStcCnt;
#endif
			}
			if (tEventFlag & (1<<MTMR_RXSLOT_OFFSET)) // rx slot offset
			{
				do_write((tmrBaseAddr+MTMR_TEVENT0_REG), (1<<MTMR_RXSLOT_OFFSET));
				do_write(tFlagAddr, (1<<MTMR_RXSLOT_OFFSET));	// clear int flag
				pMtimerInt->rxSlotIntCnt++;
				//uint32_t addr = 0;
#ifdef PALLADIUM_TEST
				debug_write((DBG_DDR_IDX_DRV_BASE+33), pMtimerInt->rxSlotIntCnt);	// 0x84
#endif
				pMtimerSfn->rxSlotNum++;
				__ucps2_synch(0);
				if (pMtimerSfn->rxSlotNum == pMtimerSfn->slotMaxNum)
				{
					pMtimerSfn->rxSfnNum++;
					pMtimerSfn->rxSfnNum &= 0x3FF;
					pMtimerSfn->rxSlotNum = 0;
				}
				__ucps2_synch(0);
			}
			tEventFlag = do_read_volatile(tFlagAddr);
		}
		do_write((tmrBaseAddr+MTMR_INTC_REG), (1 << MTMR_INT_SLOT_OFFSET));			// clear int
	}
}


