// +FHDR------------------------------------------------------------
//                 Copyright (c) 2022 SmartLogic.
//                       ALL RIGHTS RESERVED
// -----------------------------------------------------------------
// Filename      : phy_init.c
// Author        : xianfeng.du
// Created On    : 2022-12-27
// Last Modified :
// -----------------------------------------------------------------
// Description:
// ecs_rfm_spu1 phy_init
//
// -FHDR------------------------------------------------------------

#include "phy_para.h"
#include "mtimer_cell.h"
#include "ucp_utility.h"
#include "hw_cpri.h"

#define CELL_SETUP_TYPE_SIMULATION (0x5a6b7c8d)

void ecs_rfm1_build_cell(void)
{
	phy_timer_config_ind_t my_cpritmr;

#ifdef CPRI_TIMING_7D2U_TEST
    my_cpritmr.scsId = NR_SCS_30K;
#ifdef TEST_ENABLE
	my_cpritmr.runCoreId = 0;
#else
    my_cpritmr.runCoreId = 0xF; // 0x3; // 
#endif
    my_cpritmr.t_period = 5000;
    my_cpritmr.t_us = 500;
    my_cpritmr.num_t = 10;
    my_cpritmr.num_t_per_sfn = 20;

	my_cpritmr.num_t_dl = 7;		// dl slot num
	my_cpritmr.num_t_dl_symb = 6;	// dl symbol num
	my_cpritmr.num_t_ul_symb = 4;	// ul symbol num
	my_cpritmr.num_ants = 4;
#endif

#ifdef CPRI_TIMING_LTE_FDD_TEST
    my_cpritmr.scsId = LTE_SCS_ID;
#ifdef TEST_ENABLE
    my_cpritmr.runCoreId = 0;
#else
    my_cpritmr.runCoreId = 0x3; // 
#endif
    my_cpritmr.t_period = 10000;
    my_cpritmr.t_us = 1000;
    my_cpritmr.num_t = 10;
    my_cpritmr.num_t_per_sfn = 10;

   	my_cpritmr.num_t_dl =10;// 7;	   	// dl slot num
   	my_cpritmr.num_t_dl_symb = 14;//6;  // dl symbol num
   	my_cpritmr.num_t_ul_symb = 14;//4;  // ul symbol num
   	my_cpritmr.num_ants = 2;
#endif

	mtimer_init4phy(&my_cpritmr);
}

uint32_t gCellFlag = 0;
void phy_init(void)
{
	gCellFlag = 0;
	return;
}

void phy_msg_proc(uint32_t u32msg_addr, uint32_t u32msg_size)
{
	uint32_t msg_buf = do_read(u32msg_addr);
	uint32_t msg_type = do_read(msg_buf);
	if ((CELL_SETUP_TYPE_SIMULATION == msg_type) && (0 == gCellFlag)) 
	{
		ecs_rfm1_build_cell();
		gCellFlag = 1;
	}

	return;
}


uint32_t gDdrCallCnt = 0;
void ddr_wr_callback()
{
	gDdrCallCnt++;
	debug_write((DBG_DDR_IDX_DRV_BASE+127), gDdrCallCnt); // 0x1fc
}

uint32_t gFapiCallCnt = 0;
void rfm1_fapi_callback()
{
	gFapiCallCnt++;
	debug_write((DBG_DDR_IDX_DRV_BASE+123), gFapiCallCnt); // 0x1ec
}


