#ifndef __SMARTOS_H__
#define __SMARTOS_H__

typedef unsigned int    uint32_t;
typedef unsigned short  uint16_t;
typedef unsigned char   uint8_t;

typedef void (*start_hook_func)(int);
typedef void (*irq_handler)();

typedef enum
{
   smart_all_irq,
   smart_fun_s,
   smart_fun_e,
   smart_clear_cnt_error,
   smart_irq_check_error

}smart_irq_cnt_type_t;
   

/*
	ƣsmart_kernel_init
	: usr_startup
	: cpuid
	ͣvoid
	ܣsmartϵͳʼӿ

*/
extern void smart_kernel_init(start_hook_func *usr_startup,int cpuid);


/*
	ƣsmart_irq_enable
	: vector
	ͣint
	ֵ    С0 ע
	ֵ0 ɹ
	ܣ ʹܶӦжϺŵintmask

*/

int smart_irq_enable(uint32_t vector);

/*
	ƣsmart_irq_disable
	: vector
	ͣint
	ֵ    С0 ע
	ֵ0 ɹ
	ܣ ζӦжϺŶӦintmask

*/

int smart_irq_disable(uint32_t vector);


/*
	ƣsmart_irq_request
	: vector
	: func  ָ
	ͣint
	ֵ    С0 עʧ
	ֵ0 עɹ
	ܣ жעӿ

*/


extern int smart_irq_request(uint32_t vector,irq_handler func);

/*
	ƣsmart_irq_free
	vector
	ֵ0:ɹ 0ʧ
	ܣжע
*/

extern int smart_irq_free(uint32_t vector);

/*
	ƣsmart_int_disable
	
	ֵ
	ܣж
*/

extern void smart_int_disable();

/*
	ƣsmart_int_enable
	
	ֵ
	ܣж
*/

extern void smart_int_enable();



/*
	ƣsmart_get_curprio
	: 
	ͣint
	ֵ    С0ȡʧ
	ֵ0ȡɹ
*/



extern int smart_get_curprio();

/*
	ƣsmart_get_cpuid
	: 
	ͣint
	ֵ    С0ȡʧ
	ֵ0ȡɹ
*/

extern int smart_get_cpuid();

/*
	ƣsmart_task_create
	: void   (*task)(void *p_arg)
	*p_arg
	*ptos; ջ
	prio ȼ
	id id 
	*pbos ջ
	stk_size; ջС
	ͣint
	ֵ0עɹ
	ܣע
*/

extern uint8_t  smart_task_create (void   (*task)(void *p_arg),
                        void    *p_arg,
                        uint32_t  *ptos,
                        uint8_t    prio,
                        uint16_t   id,
                        uint32_t  *pbos,
                        uint32_t   stk_size);

/*
	ƣsmart_tick_sleep
	: cycle
	: int
	ֵ
	ܣtick ʱ
*/

extern void smart_tick_sleep(int cycle);

/*
	ƣsmart_task_del
	: prio
	: int
	ֵ
	ܣɾ
*/

extern void smart_task_del(int prio);

/*****************************************************************
sem relation
****************************************************************/

/******************************************
ƣsmart_sem_create
: cnt
: int
ֵvoid *
ܣźź

*****************************************/
extern void *smart_sem_create(int cnt);

/******************************************
ƣsmart_sem_pend
: void *sem ź
: void *
ֵ
ܣȡźʧ
*****************************************/

extern void smart_sem_pend(void *sem);

/******************************************
ƣsmart_sem_post
: void *sem ź
: void *
ֵ 0 ɹС0 ʧ
ܣͷź
*****************************************/

extern int smart_sem_post(void *sem);

/*******************************************

	ƣsmart_sem_del
	: void *sem ź
	: void *
	ֵ 0 ɹС0 ʧ
	ܣͷź

******************************************/
extern int smart_sem_del(void *sem);

/*
	ƣsmart_irq_debug_init
	debug_base_addr buf ׵ַ
	len_bytes buf 
	ͣint
	ֵ      С0 ʧ
	ֵ      0 سɹ
	ܣжϵԳʼӿ
*/

extern int smart_irq_debug_init(uint32_t debug_base_addr, uint32_t len_bytes);

/*
	ƣsmart_irq_get_type_cnt
	num_type
	  irq_cnt_type_t
	typedef enum
	{
	    all_irq,
		fun_s,
		fun_e,
		clear_cnt_error,
		irq_check_error

	}irq_cnt_type_t;
	
	ͣuint32_t
	ֵ      С0 ʧ
	ֵ      0 سɹ
	ܣȡжϼ
*/


extern uint32_t smart_irq_get_type_cnt(smart_irq_cnt_type_t num_type);

/*
	ƣsmart_irq_get_run_cnt
	irqnum
	
	ͣint
	ֵ      С0 ʧ
	ֵ      0 سɹ
	ܣȡжͳƼ
*/


extern int smart_irq_get_run_cnt(uint32_t irqnum);




/*
	ƣsmart_os_debug_init
	debug_base_addr buf ׵ַ
	len_bytes buf 
	ͣint
	ֵ      С0 ʧ
	ֵ      0 سɹ
	ܣжϵԳʼӿ

*/


extern int smart_os_debug_init( uint32_t debug_base_addr, uint32_t len_bytes);


#endif
